import 'active_window_error.dart';
import 'active_window_filter.dart';
import 'active_window_observer.dart';

class ActiveWindowFilterGenerator {
  ActiveWindowFilterPredicate generate(ActiveWindowFilter filter) {
    switch (filter.type) {
      case ActiveWindowFilterType.contains:
        return _contains(filter);

      default:
        return _startsWith(filter);
    }
  }

  ActiveWindowFilterPredicate contains({required ActiveWindowProperty prop, required String value, hide = true}) {
    final filter = ActiveWindowFilter(type: ActiveWindowFilterType.contains, value: value, field: prop, hide: hide);
    return _contains(filter);
  }

  ActiveWindowFilterPredicate startsWith({required ActiveWindowProperty prop, required String value, hide = true}) {
    final filter = ActiveWindowFilter(type: ActiveWindowFilterType.startsWith, value: value, field: prop, hide: hide);
    return _startsWith(filter);
  }

  ActiveWindowFilterPredicate _contains(ActiveWindowFilter filter) {
    switch (filter.field) {
      case ActiveWindowProperty.appName:
        return (i) => i?.appName?.contains(filter.value) == !filter.hide;

      case ActiveWindowProperty.bundleId:
        return (i) => i?.bundleId?.contains(filter.value) == !filter.hide;

      case ActiveWindowProperty.deviceId:
        return (i) => i?.deviceId?.contains(filter.value) == !filter.hide;

      case ActiveWindowProperty.hostName:
        return (i) => i?.hostName?.contains(filter.value) == !filter.hide;

      case ActiveWindowProperty.path:
        return (i) => i?.path?.contains(filter.value) == !filter.hide;

      case ActiveWindowProperty.title:
        return (i) => i?.title.contains(filter.value) == !filter.hide;

      case ActiveWindowProperty.userName:
        return (i) => i?.userName?.contains(filter.value) == !filter.hide;

      default:
        throw ActiveWindowError(number: 1, message: 'Could not determine filter field.');
    }
  }

  ActiveWindowFilterPredicate _startsWith(ActiveWindowFilter filter) {
    switch (filter.field) {
      case ActiveWindowProperty.appName:
        return (i) => i?.appName?.startsWith(filter.value) == !filter.hide;

      case ActiveWindowProperty.bundleId:
        return (i) => i?.bundleId?.startsWith(filter.value) == !filter.hide;

      case ActiveWindowProperty.deviceId:
        return (i) => i?.deviceId?.startsWith(filter.value) == !filter.hide;

      case ActiveWindowProperty.hostName:
        return (i) => i?.hostName?.startsWith(filter.value) == !filter.hide;

      case ActiveWindowProperty.path:
        return (i) => i?.path?.startsWith(filter.value) == !filter.hide;

      case ActiveWindowProperty.title:
        return (i) => i?.title.startsWith(filter.value) == !filter.hide;

      case ActiveWindowProperty.userName:
        return (i) => i?.userName?.startsWith(filter.value) == !filter.hide;

      default:
        throw ActiveWindowError(number: 1, message: 'Could not determine filter field.');
    }
  }
}
