import 'dart:async';

import 'active_window.dart';
import 'active_window_info.dart';

ActiveWindowObserver? v;

typedef bool ActiveWindowFilterPredicate(ActiveWindowInfo? windowInfo);

class ActiveWindowObserver {
  ActiveWindowObserver._(this._controller);

  factory ActiveWindowObserver() => ActiveWindowObserver._(StreamController.broadcast());

  final StreamController<ActiveWindowInfo?> _controller;

  Timer? _infoReaderTimer;

  ActiveWindowInfo? _windowInfo;

  final List<ActiveWindowFilterPredicate> _filter = [];

  ActiveWindowInfo? get windowInfo => _windowInfo;

  void addFilter(ActiveWindowFilterPredicate predicate) {
    _filter.add(predicate);
  }

  void removeFilter(ActiveWindowFilterPredicate predicate) {
    _filter.remove(predicate);
  }

  Future<void> start({intervalInSeconds = 5}) async {
    _infoReaderTimer?.cancel();
    _infoReaderTimer = Timer.periodic(Duration(seconds: intervalInSeconds), (timer) async {
      final value = await ActiveWindow.getActiveWindowInfo;
      _controller.add(value);
    });
  }

  void close() {
    _infoReaderTimer?.cancel();
    _infoReaderTimer = null;
    _controller.close();
  }

  StreamSubscription<ActiveWindowInfo?> listen(
    void Function(ActiveWindowInfo?)? onData, {
    Function? onError,
    void Function()? onDone,
    bool? cancelOnError,
  }) {
    return _controller.stream.where(_testFilter).distinct((a, b) => a == b).listen(
          onData,
          cancelOnError: cancelOnError,
          onDone: onDone,
          onError: onError,
        );
  }
  // void cx(){
  //   v.
  //   _controller.sink.add(event)
  //   _controller.stream.distinct();
  // }

  bool _testFilter(ActiveWindowInfo? event) {
    return !_filter.any((element) => element(event) == false);
    // for (var item in _filter) {
    //   if (item(event) == false) {
    //     return false;
    //   }
    // }
    // return true;
  }

  void addDefaultPrivateFilter() {
    _filter.addAll([
      (i) => i?.title.toLowerCase().contains('chat') == false,
      (i) => i?.title.toLowerCase().contains('urlaub') == false,
      (i) => i?.title.toLowerCase().contains('holiday') == false,
      (i) => i?.title.toLowerCase().contains('ddnss') == false,
      (i) => i?.title.toLowerCase().contains('neuer Tab') == false,
      (i) => i?.title.toLowerCase().contains('startseite') == false,
      (i) => i?.title.toLowerCase().contains('sprungliste') == false,
    ]);
  }
}
